package mwe;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.apache.commons.lang3.math.NumberUtils;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.MultiGraph;
import org.graphstream.ui.graphicGraph.GraphicGraph;
import org.graphstream.ui.graphicGraph.GraphicNode;
import org.graphstream.ui.swing_viewer.SwingViewer;
import org.graphstream.ui.view.View;
import org.graphstream.ui.view.Viewer;
import org.graphstream.ui.view.ViewerListener;
import org.graphstream.ui.view.ViewerPipe;

class NodeSyncTest {
	
	public static void main(String[] args) {
		System.out.println("NodeSyncTest.main : " + Thread.currentThread().getName());
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			@Override
            public void run() {
				System.out.println("SwingUtilities.invokeLater.Runnable.run : " + Thread.currentThread().getName());
				new NodeSyncTest();
            }
        });
	}
	
	NodeSyncTest() {
				
		Map<String, MyNode> myNodes = Map.of(
			    "fixed_top_left", new MyNode(-2, 2),
			    "fixed_top_right", new MyNode(2, 2),
			    "fixed_bottom_left", new MyNode(-2, -2),
			    "fixed_bottom_right", new MyNode(2, -2),
			    "unfixed", new MyNode(0, 0)
			);

		GraphStreamControl graphStreamControl = new GraphStreamControl(myNodes);
		
		JTextFieldsControl jTextFieldsControl = new JTextFieldsControl(myNodes);
		
		graphStreamControl.jTextFieldsControl = jTextFieldsControl;
		jTextFieldsControl.graphStreamControl = graphStreamControl;
		
		graphStreamControl.fillGraphStreamGraph();
		
		JFrame mainDialog = new JFrame();
		
		mainDialog.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainDialog.setSize(300, 300);

		mainDialog.getContentPane().add((Component) graphStreamControl.view, BorderLayout.CENTER);
		mainDialog.getContentPane().add(jTextFieldsControl.panel, BorderLayout.SOUTH);
		
		mainDialog.setLocationRelativeTo(null);
		
		mainDialog.setVisible(true);
		
		graphStreamControl.startPumpLoop();
	}
	
	 class GraphStreamControl {
		 
		 Map<String, MyNode> myNodes;
		 
		 MyNode myUnfixedNode;
		 
		 Graph graphStreamGraph;
		 
		 Viewer viewer;
		 
		 View view;
		 
		 ViewerPipe viewerPipe;
		 
		 JTextFieldsControl jTextFieldsControl;
		
		 GraphStreamControl(Map<String, MyNode> myNodes) {
			 
			this.myNodes = myNodes;
			
			myUnfixedNode = myNodes.get("unfixed");
						
			System.setProperty("org.graphstream.ui", "swing");
			
			graphStreamGraph = new MultiGraph("");
			
			viewer = new SwingViewer(graphStreamGraph, Viewer.ThreadingModel.GRAPH_IN_ANOTHER_THREAD);
			viewer.disableAutoLayout();
			view = viewer.addDefaultView(false);
			viewerPipe = viewer.newViewerPipe();
			viewerPipe.addSink(graphStreamGraph);
			
			viewerPipe.addViewerListener(new ViewerListener() {

				@Override
				public void viewClosed(String viewName) {}

				@Override
				public void buttonPushed(String id) {}

				@Override
				public void buttonReleased(String id) {
					System.out.println("ViewerListener.buttonReleased : " + Thread.currentThread().getName());
					if ("unfixed".equals(id)) {
						GraphicGraph graphicGraph = viewer.getGraphicGraph();
						GraphicNode unfixedGraphStreamNode = (GraphicNode) graphicGraph.getNode("unfixed");
						myUnfixedNode.x = unfixedGraphStreamNode.getX();
						myUnfixedNode.y = unfixedGraphStreamNode.getY();
						jTextFieldsControl.update();
					}
				}

				@Override
				public void mouseOver(String id) {}

				@Override
				public void mouseLeft(String id) {}

			});

		}
		 
		public void fillGraphStreamGraph() {
			for (var entry : myNodes.entrySet()) {
			    String nodeId = entry.getKey();
			    MyNode myNode = entry.getValue();
			    Node graphStreamNode = graphStreamGraph.addNode(nodeId);
			    graphStreamNode.setAttribute("xy", myNode.x, myNode.y);
			    graphStreamNode.setAttribute("ui.label", nodeId);
			    graphStreamNode.setAttribute("ui.style", "text-alignment: under;");
			}
			
		}

		void startPumpLoop() {
			new Thread(() -> {
				System.out.println("GraphStreamControl.startPumpLoop : " + Thread.currentThread().getName());
				while (true) {
					try {
						viewerPipe.blockingPump();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}).start();
		}

		void update() {
			GraphicGraph graphicGraph = viewer.getGraphicGraph();
			GraphicNode unfixedGraphStreamNode = (GraphicNode) graphicGraph.getNode("unfixed");
			unfixedGraphStreamNode.setAttribute("xy", myUnfixedNode.x, myUnfixedNode.y);	
		}
		
	}
	 
	class JTextFieldsControl {
		
		Map<String, MyNode> myNodes;
		
		MyNode myUnfixedNode;
		
		JPanel panel;
		
		JTextField xTextField;
		
		JTextField yTextField;
		
		GraphStreamControl graphStreamControl;
		
		JTextFieldsControl(Map<String, MyNode> myNodes) {
			
			this.myNodes = myNodes;
			
			myUnfixedNode = myNodes.get("unfixed");
									
			panel = new JPanel(new GridLayout(1, 4));
			
			JLabel xLabel = new JLabel("X:", SwingConstants.RIGHT);
			xLabel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			panel.add(xLabel);
			xTextField = new JTextField(3);
			xTextField.setHorizontalAlignment(SwingConstants.RIGHT);
			xTextField.setText(Double.toString(myUnfixedNode.x));
			xTextField.getCaret().setDot(0);
			xTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					System.out.println("JTextFieldsControl - actionPerformed on xTextField : " + Thread.currentThread().getName());
					String xNodeString = xTextField.getText();
					double xNodeDouble = NumberUtils.toDouble(xNodeString);
					myUnfixedNode.x = xNodeDouble;
					graphStreamControl.update();
					
				}
			});
			panel.add(xTextField);
			
			JLabel yLabel = new JLabel("Y:", SwingConstants.RIGHT);
			yLabel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
			panel.add(yLabel);
			yTextField = new JTextField(3);
			yTextField.setHorizontalAlignment(SwingConstants.RIGHT);
			yTextField.setText(Double.toString(myUnfixedNode.y));
			yTextField.getCaret().setDot(0);
			yTextField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent event) {
					System.out.println("JTextFieldsControl - actionPerformed on yTextField : " + Thread.currentThread().getName());
					String yNodeString = yTextField.getText();
					double yNodeDouble = NumberUtils.toDouble(yNodeString);
					myUnfixedNode.y = yNodeDouble;
					graphStreamControl.update();
				}
			});
			panel.add(yTextField);
		}
		
		void update() {
			String xNodeString = Double.toString(myUnfixedNode.x);
			xTextField.setText(xNodeString);
			xTextField.getCaret().setDot(0);
			String yNodeString = Double.toString(myUnfixedNode.y);
			yTextField.setText(yNodeString);
			yTextField.getCaret().setDot(0);
		}
	}
	
	class MyNode {
		double x;
		double y;
		MyNode(double x, double y) {
			this.x = x;
			this.y = y;
		}
	}
	
}

